@@ -115,12 +115,18 @@ adjust_colors_exec (GthAsyncTask *task,
 	unsigned char   *p_destination;
 	gboolean         cancelled;
 	double           progress;
-	int              x, y;
+	int              x, y, temp;
 	unsigned char    values[4];
 	int              channel;
 	unsigned char    value;
+	double           saturation;
 	cairo_surface_t *destination;
 
+	if (adjust_data->saturation < 0)
+		saturation = tan (adjust_data->saturation * G_PI_2);
+	else
+		saturation = adjust_data->saturation;
+
 	source = gth_image_task_get_source_surface (GTH_IMAGE_TASK (task));
 	format = cairo_image_surface_get_format (source);
 	width = cairo_image_surface_get_width (source);
@@ -152,7 +158,8 @@ adjust_colors_exec (GthAsyncTask *task,
 				if (! pixbuf_cache_get (adjust_data->cache, channel + 1, &value)) {
 					int tmp;
 
-					value = gamma_correction (value, adjust_data->gamma);
+					if (adjust_data->gamma != 0.0)
+						value = gamma_correction (value, adjust_data->gamma);
 
 					if (adjust_data->brightness > 0)
 						tmp = interpolate_value (value, 0, adjust_data->brightness);
@@ -179,18 +186,12 @@ adjust_colors_exec (GthAsyncTask *task,
 
 			if (adjust_data->saturation != 0.0) {
 				guchar min, max, lightness;
-				double saturation;
 				int    tmp;
 
 				max = MAX (MAX (values[0], values[1]), values[2]);
 				min = MIN (MIN (values[0], values[1]), values[2]);
 				lightness = (max + min) / 2;
 
-				if (adjust_data->saturation < 0)
-					saturation = tan (adjust_data->saturation * G_PI_2);
-				else
-					saturation = adjust_data->saturation;
-
 				tmp = interpolate_value (values[0], lightness, saturation);
 				values[0] = CLAMP (tmp, 0, 255);
 
@@ -231,20 +232,6 @@ adjust_data_free (gpointer user_data)
 }
 
 
-static void apply_changes (GthFileToolAdjustColors *self);
-
-
-static void
-ok_button_clicked_cb (GtkButton *button,
-		      gpointer   user_data)
-{
-	GthFileToolAdjustColors *self = user_data;
-
-	self->priv->apply_to_original = TRUE;
-	apply_changes (self);
-}
-
-
 static void
 reset_button_clicked_cb (GtkButton *button,
 		  	 gpointer   user_data)
@@ -261,6 +248,9 @@ reset_button_clicked_cb (GtkButton *button,
 }
 
 
+static void apply_changes (GthFileToolAdjustColors *self);
+
+
 static void
 image_task_completed_cb (GthTask  *task,
 			 GError   *error,
@@ -406,18 +396,15 @@ gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 	GtkWidget               *options;
 	int                      width, height;
 	GtkAllocation            allocation;
-	gboolean                 rtl;
 
 	self = (GthFileToolAdjustColors *) base;
 
-	rtl = gtk_widget_get_direction (GTK_WIDGET (base)) == GTK_TEXT_DIR_RTL;
-
 	viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
 	if (viewer_page == NULL)
 		return NULL;
 
-	cairo_surface_destroy (self->priv->destination);
-	cairo_surface_destroy (self->priv->preview);
+	_cairo_clear_surface (&self->priv->destination);
+	_cairo_clear_surface (&self->priv->preview);
 
 	source = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
 	if (source == NULL)
@@ -428,7 +415,7 @@ gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
 	if (scale_keeping_ratio (&width, &height, PREVIEW_SIZE * allocation.width, PREVIEW_SIZE * allocation.height, FALSE))
-		self->priv->preview = _cairo_image_surface_scale_bilinear (source, width, height);
+		self->priv->preview = _cairo_image_surface_scale_fast (source, width, height);
 	else
 		self->priv->preview = cairo_surface_reference (source);
 
@@ -440,10 +427,6 @@ gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 	options = _gtk_builder_get_widget (self->priv->builder, "options");
 	gtk_widget_show (options);
 
-	gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET("reset_image")), rtl ? "edit-undo-rtl-symbolic" :
-										   "edit-undo-symbolic",
-									     GTK_ICON_SIZE_MENU);
-
 	self->priv->histogram_view = gth_histogram_view_new (self->priv->histogram);
 	gtk_widget_show (self->priv->histogram_view);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("histogram_hbox")), self->priv->histogram_view, TRUE, TRUE, 0);
@@ -477,18 +460,6 @@ gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 								   GTH_COLOR_SCALE_YELLOW_BLUE,
 								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
 
-	g_signal_connect (GET_WIDGET ("ok_button"),
-			  "clicked",
-			  G_CALLBACK (ok_button_clicked_cb),
-			  self);
-	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
-				  "clicked",
-				  G_CALLBACK (gth_file_tool_cancel),
-				  self);
-	g_signal_connect (GET_WIDGET ("reset_button"),
-			  "clicked",
-			  G_CALLBACK (reset_button_clicked_cb),
-			  self);
 	g_signal_connect (G_OBJECT (self->priv->brightness_adj),
 			  "value-changed",
 			  G_CALLBACK (value_changed_cb),
@@ -554,6 +525,43 @@ gth_file_tool_adjust_colors_destroy_options (GthFileTool *base)
 }
 
 
+static void
+gth_file_tool_adjust_colors_apply_options (GthFileTool *base)
+{
+	GthFileToolAdjustColors *self;
+
+	self = (GthFileToolAdjustColors *) base;
+	self->priv->apply_to_original = TRUE;
+	apply_changes (self);
+}
+
+
+static void
+gth_file_tool_adjust_colors_populate_headerbar (GthFileTool *base,
+						GthBrowser  *browser)
+{
+	GthFileToolAdjustColors *self;
+	gboolean                 rtl;
+	GtkWidget               *button;
+
+	self = (GthFileToolAdjustColors *) base;
+
+	/* reset button */
+
+	rtl = gtk_widget_get_direction (GTK_WIDGET (base)) == GTK_TEXT_DIR_RTL;
+	button = gth_browser_add_header_bar_button (browser,
+						    GTH_BROWSER_HEADER_SECTION_EDITOR_COMMANDS,
+						    rtl ? "edit-undo-rtl-symbolic" : "edit-undo-symbolic",
+						    _("Reset"),
+						    NULL,
+						    NULL);
+	g_signal_connect (button,
+			  "clicked",
+			  G_CALLBACK (reset_button_clicked_cb),
+			  self);
+}
+
+
 static void
 gth_file_tool_sharpen_reset_image (GthImageViewerPageTool *base)
 {
@@ -624,6 +632,8 @@ gth_file_tool_adjust_colors_class_init (GthFileToolAdjustColorsClass *klass)
 	file_tool_class = (GthFileToolClass *) klass;
 	file_tool_class->get_options = gth_file_tool_adjust_colors_get_options;
 	file_tool_class->destroy_options = gth_file_tool_adjust_colors_destroy_options;
+	file_tool_class->apply_options = gth_file_tool_adjust_colors_apply_options;
+	file_tool_class->populate_headerbar = gth_file_tool_adjust_colors_populate_headerbar;
 
 	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
 	image_viewer_page_tool_class->reset_image = gth_file_tool_sharpen_reset_image;
