@@ -62,15 +62,6 @@ struct _GthFileToolResizePrivate {
 static void update_image_size (GthFileToolResize *self);
 
 
-static void
-resize_button_clicked_cb (GtkButton       *button,
-			GthFileToolResize *self)
-{
-	self->priv->apply_to_original = TRUE;
-	update_image_size (self);
-}
-
-
 static void
 update_dimensione_info_label (GthFileToolResize *self,
 			      const char        *id,
@@ -126,10 +117,10 @@ resize_task_completed_cb (GthTask  *task,
 	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->new_image, FALSE);
 
 	if (self->priv->apply_to_original) {
-		gth_image_history_add_image (gth_image_viewer_page_get_history (GTH_IMAGE_VIEWER_PAGE (viewer_page)),
-					     self->priv->new_image,
-					     -1,
-					     TRUE);
+		gth_image_history_add_surface (gth_image_viewer_page_get_history (GTH_IMAGE_VIEWER_PAGE (viewer_page)),
+					       self->priv->new_image,
+					       -1,
+					       TRUE);
 		gth_viewer_page_focus (GTH_VIEWER_PAGE (viewer_page));
 		gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 	}
@@ -545,7 +536,7 @@ gth_file_tool_resize_get_options (GthFileTool *base)
 	preview_width = self->priv->original_width;
 	preview_height = self->priv->original_height;
 	if (scale_keeping_ratio (&preview_width, &preview_height, allocation.width, allocation.height, FALSE))
-		self->priv->preview = _cairo_image_surface_scale_bilinear (source, preview_width, preview_height);
+		self->priv->preview = _cairo_image_surface_scale_fast (source, preview_width, preview_height);
 	else
 		self->priv->preview = cairo_surface_reference (source);
 
@@ -618,18 +609,6 @@ gth_file_tool_resize_get_options (GthFileTool *base)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("ratio_h_spinbutton")),
 				   MAX (g_settings_get_int (self->priv->settings, PREF_RESIZE_ASPECT_RATIO_HEIGHT), 1));
 
-	g_signal_connect (GET_WIDGET ("resize_button"),
-			  "clicked",
-			  G_CALLBACK (resize_button_clicked_cb),
-			  self);
-	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
-				  "clicked",
-				  G_CALLBACK (gth_file_tool_cancel),
-				  self);
-	g_signal_connect (GET_WIDGET ("options_button"),
-			  "clicked",
-			  G_CALLBACK (options_button_clicked_cb),
-			  self);
 	g_signal_connect_swapped (GET_WIDGET ("options_close_button"),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_hide),
@@ -741,6 +720,43 @@ gth_file_tool_resize_destroy_options (GthFileTool *base)
 }
 
 
+static void
+gth_file_tool_resize_apply_options (GthFileTool *base)
+{
+	GthFileToolResize *self;
+
+	self = (GthFileToolResize *) base;
+
+	self->priv->apply_to_original = TRUE;
+	update_image_size (self);
+}
+
+
+static void
+gth_file_tool_resize_populate_headerbar (GthFileTool *base,
+					 GthBrowser  *browser)
+{
+	GthFileToolResize *self;
+	GtkWidget         *button;
+
+	self = (GthFileToolResize *) base;
+
+	/* preferences dialog */
+
+	button = gth_browser_add_header_bar_button (browser,
+						    GTH_BROWSER_HEADER_SECTION_EDITOR_COMMANDS,
+						    "preferences-system-symbolic",
+						    _("Options"),
+						    NULL,
+						    NULL);
+	g_signal_connect (button,
+			  "clicked",
+			  G_CALLBACK (options_button_clicked_cb),
+			  self);
+
+}
+
+
 static void
 gth_file_tool_resize_reset_image (GthImageViewerPageTool *base)
 {
@@ -797,6 +813,8 @@ gth_file_tool_resize_class_init (GthFileToolResizeClass *klass)
 	file_tool_class = (GthFileToolClass *) klass;
 	file_tool_class->get_options = gth_file_tool_resize_get_options;
 	file_tool_class->destroy_options = gth_file_tool_resize_destroy_options;
+	file_tool_class->apply_options = gth_file_tool_resize_apply_options;
+	file_tool_class->populate_headerbar = gth_file_tool_resize_populate_headerbar;
 
 	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
 	image_viewer_page_tool_class->reset_image = gth_file_tool_resize_reset_image;
