@@ -42,7 +42,8 @@ struct _GthFileToolSharpenPrivate {
 	GtkWidget       *preview;
 	guint            apply_event;
 	gboolean         show_preview;
-	gboolean         first_allocation;
+	cairo_surface_t *preview_source;
+	gboolean         image_centered;
 };
 
 
@@ -99,41 +100,6 @@ sharpen_exec (GthAsyncTask *task,
 }
 
 
-static void
-ok_button_clicked_cb (GtkButton          *button,
-		      GthFileToolSharpen *self)
-{
-	GtkWidget   *viewer_page;
-	SharpenData *sharpen_data;
-	GthTask     *task;
-
-	if (self->priv->apply_event != 0) {
-		g_source_remove (self->priv->apply_event);
-		self->priv->apply_event = 0;
-	}
-
-	viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
-	if (viewer_page == NULL)
-		return;
-
-	sharpen_data = sharpen_data_new (self);
-	task = gth_image_viewer_task_new (GTH_IMAGE_VIEWER_PAGE (viewer_page),
-					  _("Sharpening image"),
-					  NULL,
-					  sharpen_exec,
-					  NULL,
-					  sharpen_data,
-					  sharpen_data_free);
-	g_signal_connect (task,
-			  "completed",
-			  G_CALLBACK (gth_image_viewer_task_set_destination),
-			  NULL);
-	gth_browser_exec_task (GTH_BROWSER (gth_file_tool_get_window (GTH_FILE_TOOL (self))), task, FALSE);
-
-	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
-}
-
-
 static void
 reset_button_clicked_cb (GtkButton          *button,
 			 GthFileToolSharpen *self)
@@ -156,10 +122,9 @@ apply_cb (gpointer user_data)
 		self->priv->apply_event = 0;
 	}
 
-	source = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	source = self->priv->preview_source;
 	preview = GTH_IMAGE_VIEWER (self->priv->preview);
 	if (self->priv->show_preview) {
-
 		SharpenData     *sharpen_data;
 		int              x, y, w ,h;
 		cairo_surface_t *destination;
@@ -232,17 +197,17 @@ preview_checkbutton_toggled_cb (GtkToggleButton    *toggle_button,
 
 
 static void
-preview_size_allocate_cb (GtkWidget    *widget,
-			  GdkRectangle *allocation,
-			  gpointer      user_data)
+preview_site_allocate_cb (GtkWidget    *widget,
+	                  GdkRectangle *allocation,
+	                  gpointer      user_data)
 {
 	GthFileToolSharpen *self = user_data;
 
-	if (! self->priv->first_allocation)
+	if (self->priv->image_centered)
 		return;
-	self->priv->first_allocation = FALSE;
 
 	gth_image_viewer_scroll_to_center (GTH_IMAGE_VIEWER (self->priv->preview));
+	self->priv->image_centered = TRUE;
 }
 
 
@@ -253,32 +218,28 @@ gth_file_tool_sharpen_get_options (GthFileTool *base)
 	cairo_surface_t    *source;
 	GtkWidget          *options;
 	GtkWidget          *image_navigator;
-	gboolean            rtl;
 
 	self = (GthFileToolSharpen *) base;
 
-	rtl = gtk_widget_get_direction (GTK_WIDGET (base)) == GTK_TEXT_DIR_RTL;
+	_cairo_clear_surface (&self->priv->preview_source);
 
 	source = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
 	if (source == NULL)
 		return NULL;
 
-	self->priv->first_allocation = TRUE;
+	self->priv->preview_source = cairo_surface_reference (source);
 	self->priv->builder = _gtk_builder_new_from_file ("sharpen-options.ui", "file_tools");
+	self->priv->image_centered = FALSE;
 	options = _gtk_builder_get_widget (self->priv->builder, "options");
 	gtk_widget_show (options);
 
-	gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET("reset_image")), rtl ? "edit-undo-rtl-symbolic" :
-										   "edit-undo-symbolic",
-									     GTK_ICON_SIZE_MENU);
-
 	self->priv->preview = gth_image_viewer_new ();
 	gth_image_viewer_set_reset_scrollbars (GTH_IMAGE_VIEWER (self->priv->preview), FALSE);
 	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->preview), GTH_FIT_NONE);
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->preview), GTH_ZOOM_CHANGE_KEEP_PREV);
 	gth_image_viewer_set_zoom (GTH_IMAGE_VIEWER (self->priv->preview), 1.0);
 	gth_image_viewer_set_zoom_enabled (GTH_IMAGE_VIEWER (self->priv->preview), FALSE);
-	gth_image_viewer_set_surface (GTH_IMAGE_VIEWER (self->priv->preview), source, -1, -1);
+	gth_image_viewer_set_surface (GTH_IMAGE_VIEWER (self->priv->preview), self->priv->preview_source, -1, -1);
 	image_navigator = gth_image_navigator_new (GTH_IMAGE_VIEWER (self->priv->preview));
 	gtk_widget_show_all (image_navigator);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("preview_hbox")), image_navigator, TRUE, TRUE, 0);
@@ -296,18 +257,6 @@ gth_file_tool_sharpen_get_options (GthFileTool *base)
 							       GTH_COLOR_SCALE_DEFAULT,
 							       DEFAULT_THRESHOLD, 0.0, 255.0, 1.0, 1.0, "%.0f");
 
-	g_signal_connect (GET_WIDGET ("ok_button"),
-			  "clicked",
-			  G_CALLBACK (ok_button_clicked_cb),
-			  self);
-	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
-				  "clicked",
-				  G_CALLBACK (gth_file_tool_cancel),
-				  self);
-	g_signal_connect (GET_WIDGET ("reset_button"),
-			  "clicked",
-			  G_CALLBACK (reset_button_clicked_cb),
-			  self);
 	g_signal_connect (G_OBJECT (self->priv->radius_adj),
 			  "value-changed",
 			  G_CALLBACK (value_changed_cb),
@@ -332,10 +281,12 @@ gth_file_tool_sharpen_get_options (GthFileTool *base)
 			  "clicked",
 			  G_CALLBACK (preview_checkbutton_toggled_cb),
 			  self);
-	g_signal_connect_after (self->priv->preview,
-				"size-allocate",
-				G_CALLBACK (preview_size_allocate_cb),
-				self);
+	g_signal_connect (self->priv->preview,
+			  "size-allocate",
+			  G_CALLBACK (preview_site_allocate_cb),
+			  self);
+
+	cairo_surface_destroy (source);
 
 	return options;
 }
@@ -353,10 +304,74 @@ gth_file_tool_sharpen_destroy_options (GthFileTool *base)
 		self->priv->apply_event = 0;
 	}
 
+	_cairo_clear_surface (&self->priv->preview_source);
 	_g_clear_object (&self->priv->builder);
 }
 
 
+static void
+gth_file_tool_sharpen_apply_options (GthFileTool *base)
+{
+	GthFileToolSharpen *self;
+	GtkWidget          *viewer_page;
+	SharpenData        *sharpen_data;
+	GthTask            *task;
+
+	self = (GthFileToolSharpen *) base;
+
+	if (self->priv->apply_event != 0) {
+		g_source_remove (self->priv->apply_event);
+		self->priv->apply_event = 0;
+	}
+
+	viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	if (viewer_page == NULL)
+		return;
+
+	sharpen_data = sharpen_data_new (self);
+	task = gth_image_viewer_task_new (GTH_IMAGE_VIEWER_PAGE (viewer_page),
+					  _("Sharpening image"),
+					  NULL,
+					  sharpen_exec,
+					  NULL,
+					  sharpen_data,
+					  sharpen_data_free);
+	g_signal_connect (task,
+			  "completed",
+			  G_CALLBACK (gth_image_viewer_task_set_destination),
+			  NULL);
+	gth_browser_exec_task (GTH_BROWSER (gth_file_tool_get_window (GTH_FILE_TOOL (self))), task, FALSE);
+
+	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+}
+
+
+static void
+gth_file_tool_sharpen_populate_headerbar (GthFileTool *base,
+					  GthBrowser  *browser)
+{
+	GthFileToolSharpen *self;
+	gboolean            rtl;
+	GtkWidget          *button;
+
+	self = (GthFileToolSharpen *) base;
+
+	/* reset button */
+
+	rtl = gtk_widget_get_direction (GTK_WIDGET (base)) == GTK_TEXT_DIR_RTL;
+	button = gth_browser_add_header_bar_button (browser,
+						    GTH_BROWSER_HEADER_SECTION_EDITOR_COMMANDS,
+						    rtl ? "edit-undo-rtl-symbolic" : "edit-undo-symbolic",
+						    _("Reset"),
+						    NULL,
+						    NULL);
+	g_signal_connect (button,
+			  "clicked",
+			  G_CALLBACK (reset_button_clicked_cb),
+			  self);
+}
+
+
 static void
 gth_file_tool_sharpen_reset_image (GthImageViewerPageTool *base)
 {
@@ -402,6 +417,8 @@ gth_file_tool_sharpen_class_init (GthFileToolSharpenClass *klass)
 	file_tool_class = (GthFileToolClass *) klass;
 	file_tool_class->get_options = gth_file_tool_sharpen_get_options;
 	file_tool_class->destroy_options = gth_file_tool_sharpen_destroy_options;
+	file_tool_class->apply_options = gth_file_tool_sharpen_apply_options;
+	file_tool_class->populate_headerbar = gth_file_tool_sharpen_populate_headerbar;
 
 	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
 	image_viewer_page_tool_class->reset_image = gth_file_tool_sharpen_reset_image;
@@ -414,6 +431,7 @@ gth_file_tool_sharpen_init (GthFileToolSharpen *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_FILE_TOOL_SHARPEN, GthFileToolSharpenPrivate);
 	self->priv->builder = NULL;
 	self->priv->show_preview = TRUE;
+	self->priv->preview_source = NULL;
 
 	gth_file_tool_construct (GTH_FILE_TOOL (self),
 				 "image-sharpen-symbolic",
