@@ -28,11 +28,13 @@
 #include "preferences.h"
 
 
-#define UPDATE_QUALITY_DELAY 500
+#define UPDATE_QUALITY_DELAY 200
 #define UPDATE_VISIBILITY_DELAY 100
-#define N_HEADER_BAR_BUTTONS 3
+#define N_HEADER_BAR_BUTTONS 4
 #define HIDE_OVERVIEW_TIMEOUT 2 /* in seconds */
 #define OVERLAY_MARGIN 10
+#define APPLY_ICC_PROFILE_BUTTON 3
+#undef ALWAYS_LOAD_ORIGINAL_SIZE
 
 
 static void gth_viewer_page_interface_init (GthViewerPageInterface *iface);
@@ -55,6 +57,7 @@ static const GActionEntry actions[] = {
 	{ "image-redo", gth_browser_activate_image_redo },
 	{ "copy-image", gth_browser_activate_copy_image },
 	{ "paste-image", gth_browser_activate_paste_image },
+	{ "apply-icc-profile", toggle_action_activated, NULL, "true", gth_browser_activate_apply_icc_profile },
 };
 
 
@@ -95,6 +98,7 @@ struct _GthImageViewerPagePrivate {
 	gboolean           pointer_on_viewer;
 	gboolean           pointer_on_overview;
 	guint              hide_overview_id;
+	gboolean           apply_icc_profile;
 };
 
 
@@ -206,6 +210,10 @@ different_quality_ready_cb (GObject		*source_object,
 	int		    original_width;
 	int		    original_height;
 	GError		   *error = NULL;
+	cairo_surface_t    *s1;
+	cairo_surface_t    *s2;
+	int                 w1, h1, w2, h2;
+	gboolean            got_better_quality;
 
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
@@ -227,31 +235,26 @@ different_quality_ready_cb (GObject		*source_object,
 		goto clear_data;
 
 	/* check whether the image is of different quality */
-	{
-		cairo_surface_t *s1;
-		cairo_surface_t *s2;
-		int              w1, h1, w2, h2;
 
-		s1 = gth_image_get_cairo_surface (image);
-		s2 = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer));
-		w1 = cairo_image_surface_get_width (s1);
-		h1 = cairo_image_surface_get_height (s1);
-		w2 = cairo_image_surface_get_width (s2);
-		h2 = cairo_image_surface_get_height (s2);
+	s1 = gth_image_get_cairo_surface (image);
+	s2 = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer));
+	w1 = cairo_image_surface_get_width (s1);
+	h1 = cairo_image_surface_get_height (s1);
+	w2 = cairo_image_surface_get_width (s2);
+	h2 = cairo_image_surface_get_height (s2);
+	got_better_quality = ((w1 > w2) || (h1 > h2));
 
-		cairo_surface_destroy (s1);
-
-		if ((w1 == w2) && (h1 == h2))
-			goto clear_data;
+	if (got_better_quality) {
+		gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
+		gth_image_viewer_set_better_quality (GTH_IMAGE_VIEWER (self->priv->viewer),
+						     image,
+						     original_width,
+						     original_height);
+		gth_image_viewer_set_requested_size (GTH_IMAGE_VIEWER (self->priv->viewer), requested_size);
+		gtk_widget_queue_draw (self->priv->viewer);
 	}
 
-	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
-	gth_image_viewer_set_better_quality (GTH_IMAGE_VIEWER (self->priv->viewer),
-					     image,
-					     original_width,
-					     original_height);
-	gth_image_viewer_set_requested_size (GTH_IMAGE_VIEWER (self->priv->viewer), requested_size);
-	gtk_widget_queue_draw (self->priv->viewer);
+	cairo_surface_destroy (s1);
 
 clear_data:
 
@@ -292,6 +295,16 @@ _g_mime_type_can_load_different_quality (const char *mime_type)
 }
 
 
+static void
+_gth_image_preloader_init_preloader (GthImageViewerPage *self)
+{
+	if (self->priv->apply_icc_profile)
+		gth_image_preloader_set_out_profile (self->priv->preloader, gth_browser_get_screen_profile (self->priv->browser));
+	else
+		gth_image_preloader_set_out_profile (self->priv->preloader, NULL);
+}
+
+
 static gboolean
 update_quality_cb (gpointer user_data)
 {
@@ -308,6 +321,7 @@ update_quality_cb (gpointer user_data)
 	if (! self->priv->image_changed && ! _g_mime_type_can_load_different_quality (gth_file_data_get_mime_type (self->priv->file_data)))
 		return FALSE;
 
+	_gth_image_preloader_init_preloader (self);
 	gth_image_preloader_load (self->priv->preloader,
 				  self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
 				  _gth_image_preloader_get_requested_size_for_current_image (self),
@@ -324,6 +338,7 @@ update_quality_cb (gpointer user_data)
 static void
 update_image_quality_if_required (GthImageViewerPage *self)
 {
+#ifndef ALWAYS_LOAD_ORIGINAL_SIZE
 	GthImage *image;
 
 	if (self->priv->loading_image || gth_sidebar_tool_is_active (GTH_SIDEBAR (gth_browser_get_viewer_sidebar (self->priv->browser))))
@@ -341,6 +356,7 @@ update_image_quality_if_required (GthImageViewerPage *self)
 	self->priv->update_quality_id = g_timeout_add (UPDATE_QUALITY_DELAY,
 						       update_quality_cb,
 						       self);
+#endif
 }
 
 
@@ -439,6 +455,7 @@ viewer_image_changed_cb (GtkWidget          *widget,
 			 GthImageViewerPage *self)
 {
 	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
+	update_image_quality_if_required (self);
 	update_overview_visibility (self);
 	update_zoom_info (self);
 }
@@ -842,6 +859,14 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 							   _("Fit to width"),
 							   "win.image-zoom-fit-width",
 							   NULL);
+	self->priv->buttons[APPLY_ICC_PROFILE_BUTTON] =
+			gth_browser_add_header_bar_toggle_button (browser,
+							   	  GTH_BROWSER_HEADER_SECTION_VIEWER_TOOLS,
+								  "color-profile",
+								  _("Apply the embedded color profile"),
+								  "win.apply-icc-profile",
+								  NULL);
+
 	gth_window_add_accelerators (GTH_WINDOW (browser), accelerators, G_N_ELEMENTS (accelerators));
 
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
@@ -1039,7 +1064,7 @@ preloader_load_ready_cb (GObject	*source_object,
 
 	gth_image_history_clear (self->priv->history);
 	gth_image_history_add_image (self->priv->history,
-				     gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer)),
+				     image,
 				     requested_size,
 				     FALSE);
 
@@ -1069,7 +1094,71 @@ clear_data:
 }
 
 
-#define N_PRELOADERS 4
+#define N_FORWARD_PRELOADERS 2
+#define N_BACKWARD_PRELOADERS 2
+
+
+static void
+_gth_image_viewer_page_load (GthImageViewerPage *self,
+		             GthFileData        *file_data)
+{
+	GthFileStore *file_store;
+	GtkTreeIter   iter;
+	int           i;
+	GthFileData  *next_file_data[N_FORWARD_PRELOADERS];
+	GthFileData  *prev_file_data[N_BACKWARD_PRELOADERS];
+
+	if (self->priv->file_data != file_data) {
+		_g_object_unref (self->priv->file_data);
+		self->priv->file_data = gth_file_data_dup (file_data);
+	}
+	self->priv->image_changed = FALSE;
+	self->priv->loading_image = TRUE;
+
+	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
+		next_file_data[i] = NULL;
+	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
+		prev_file_data[i] = NULL;
+
+	file_store = gth_browser_get_file_store (self->priv->browser);
+	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
+		GtkTreeIter next_iter;
+
+		next_iter = iter;
+		for (i = 0; i < N_FORWARD_PRELOADERS; i++) {
+			if (! gth_file_store_get_next_visible (file_store, &next_iter))
+				break;
+			next_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+		}
+
+		next_iter = iter;
+		for (i = 0; i < N_BACKWARD_PRELOADERS; i++) {
+			if (! gth_file_store_get_prev_visible (file_store, &next_iter))
+				break;
+			prev_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+		}
+
+		gth_image_viewer_set_void (GTH_IMAGE_VIEWER (self->priv->viewer));
+	}
+
+	_gth_image_preloader_init_preloader (self);
+	gth_image_preloader_load (self->priv->preloader,
+				  self->priv->file_data,
+#ifdef ALWAYS_LOAD_ORIGINAL_SIZE
+				  GTH_ORIGINAL_SIZE,
+#else
+				  _gth_image_preloader_get_requested_size_for_next_images (self),
+#endif
+				  NULL,
+				  preloader_load_ready_cb,
+				  self,
+				  N_FORWARD_PRELOADERS + N_BACKWARD_PRELOADERS,
+				  next_file_data[0],
+				  next_file_data[1],
+				  prev_file_data[0],
+				  prev_file_data[1]);
+
+}
 
 
 static void
@@ -1077,11 +1166,6 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 				 GthFileData   *file_data)
 {
 	GthImageViewerPage *self;
-	GthFileStore       *file_store;
-	GtkTreeIter         iter;
-	int                 i;
-	GthFileData        *next_file_data[N_PRELOADERS];
-	GthFileData        *prev_file_data[N_PRELOADERS];
 
 	self = (GthImageViewerPage*) base;
 	g_return_if_fail (file_data != NULL);
@@ -1099,82 +1183,7 @@ gth_image_viewer_page_real_view (GthViewerPage *base,
 		return;
 	}
 
-	_g_object_unref (self->priv->file_data);
-	self->priv->file_data = gth_file_data_dup (file_data);
-	self->priv->image_changed = FALSE;
-	self->priv->loading_image = TRUE;
-
-	for (i = 0; i < N_PRELOADERS; i++) {
-		next_file_data[i] = NULL;
-		prev_file_data[i] = NULL;
-	}
-
-	file_store = gth_browser_get_file_store (self->priv->browser);
-	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
-		GtkTreeIter      next_iter;
-		gboolean         thumbnail_available;
-		cairo_surface_t *thumbnail;
-
-		next_iter = iter;
-		for (i = 0; i < N_PRELOADERS; i++) {
-			if (! gth_file_store_get_next_visible (file_store, &next_iter))
-				break;
-			next_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
-		}
-
-		next_iter = iter;
-		for (i = 0; i < N_PRELOADERS; i++) {
-			if (! gth_file_store_get_prev_visible (file_store, &next_iter))
-				break;
-			prev_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
-		}
-
-		thumbnail_available = FALSE;
-		gtk_tree_model_get (GTK_TREE_MODEL (file_store),
-				    &iter,
-				    GTH_FILE_STORE_THUMBNAIL_COLUMN,
-				    &thumbnail,
-				    -1);
-
-		if (thumbnail != NULL) {
-			cairo_surface_metadata_t *metadata;
-			int                       original_width;
-			int                       original_height;
-
-			metadata = _cairo_image_surface_get_metadata (thumbnail);
-			original_width = metadata->thumbnail.image_width;
-			original_height = metadata->thumbnail.image_height;
-
-			if ((thumbnail != NULL) && (original_width > 0) && (original_height > 0)) {
-				gth_image_viewer_set_surface (GTH_IMAGE_VIEWER (self->priv->viewer),
-							      thumbnail,
-							      original_width,
-							      original_height);
-				thumbnail_available = TRUE;
-			}
-
-			cairo_surface_destroy (thumbnail);
-		}
-
-		if (! thumbnail_available)
-			gth_image_viewer_set_void (GTH_IMAGE_VIEWER (self->priv->viewer));
-	}
-
-	gth_image_preloader_load (self->priv->preloader,
-				  self->priv->file_data,
-				  _gth_image_preloader_get_requested_size_for_next_images (self),
-				  NULL,
-				  preloader_load_ready_cb,
-				  self,
-				  N_PRELOADERS * 2,
-				  next_file_data[0],
-				  next_file_data[1],
-				  next_file_data[2],
-				  next_file_data[3],
-				  prev_file_data[0],
-				  prev_file_data[1],
-				  prev_file_data[2],
-				  prev_file_data[3]);
+	_gth_image_viewer_page_load (self, file_data);
 }
 
 
@@ -1227,6 +1236,7 @@ gth_image_viewer_page_real_update_sensitivity (GthViewerPage *base)
 	gboolean            zoom_enabled;
 	double              zoom;
 	GthFit              fit_mode;
+	GthImage           *image;
 
 	self = (GthImageViewerPage*) base;
 
@@ -1244,6 +1254,10 @@ gth_image_viewer_page_real_update_sensitivity (GthViewerPage *base)
 	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit", zoom_enabled && (fit_mode != GTH_FIT_SIZE));
 	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "image-zoom-fit-width", zoom_enabled && (fit_mode != GTH_FIT_WIDTH));
 
+	image = gth_image_viewer_get_image (GTH_IMAGE_VIEWER (self->priv->viewer));
+	gtk_widget_set_visible (self->priv->buttons[APPLY_ICC_PROFILE_BUTTON], (image != NULL) && (gth_image_get_icc_profile (image) != NULL));
+	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "apply-icc-profile", (image != NULL) && (gth_image_get_icc_profile (image) != NULL));
+
 	_gth_image_viewer_page_update_paste_command_sensitivity (self, NULL);
 }
 
@@ -1491,21 +1505,20 @@ gth_image_viewer_page_real_save_as (GthViewerPage *base,
 
 static void
 _gth_image_viewer_page_set_image (GthImageViewerPage *self,
-				  cairo_surface_t    *image,
+				  GthImage           *image,
 				  int                 requested_size,
 				  gboolean            modified)
 {
 	GthFileData *file_data;
 	int          width;
 	int          height;
-	char        *size;
 
 	if (image == NULL)
 		return;
 
 	if (modified)
 		gth_image_preloader_set_modified_image (self->priv->preloader, image);
-	gth_image_viewer_set_surface (GTH_IMAGE_VIEWER (self->priv->viewer), image, -1, -1);
+	gth_image_viewer_set_image (GTH_IMAGE_VIEWER (self->priv->viewer), image, -1, -1);
 	gth_image_viewer_set_requested_size (GTH_IMAGE_VIEWER (self->priv->viewer), requested_size);
 
 	file_data = gth_browser_get_current_file (GTH_BROWSER (self->priv->browser));
@@ -1513,16 +1526,16 @@ _gth_image_viewer_page_set_image (GthImageViewerPage *self,
 	self->priv->image_changed = modified;
 	g_file_info_set_attribute_boolean (file_data->info, "gth::file::is-modified", modified);
 
-	if (! _cairo_image_surface_get_original_size (image, &width, &height)) {
-		width = cairo_image_surface_get_width (image);
-		height = cairo_image_surface_get_height (image);
-	}
-	g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
-	g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
+	if (gth_image_get_original_size (image, &width, &height)) {
+		char *size;
 
-	size = g_strdup_printf (_("%d × %d"), width, height);
-	g_file_info_set_attribute_string (file_data->info, "general::dimensions", size);
-	g_free (size);
+		g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
+		g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
+
+		size = g_strdup_printf (_("%d × %d"), width, height);
+		g_file_info_set_attribute_string (file_data->info, "general::dimensions", size);
+		g_free (size);
+	}
 
 	gth_monitor_metadata_changed (gth_main_get_default_monitor (), file_data);
 
@@ -1530,6 +1543,21 @@ _gth_image_viewer_page_set_image (GthImageViewerPage *self,
 }
 
 
+static void
+_gth_image_viewer_page_set_surface (GthImageViewerPage *self,
+				    cairo_surface_t    *surface,
+				    int                 requested_size,
+				    gboolean            modified)
+{
+	GthImage *image;
+
+	image = gth_image_new_for_surface (surface);
+	_gth_image_viewer_page_set_image (self, image, requested_size, modified);
+
+	g_object_unref (image);
+}
+
+
 static void
 gth_image_viewer_page_real_revert (GthViewerPage *base)
 {
@@ -1661,6 +1689,7 @@ gth_image_viewer_page_init (GthImageViewerPage *self)
 	self->priv->pointer_on_overview = FALSE;
 	self->priv->pointer_on_viewer = FALSE;
 	self->priv->hide_overview_id = 0;
+	self->priv->apply_icc_profile = TRUE;
 }
 
 
@@ -1715,9 +1744,9 @@ gth_image_viewer_page_set_image (GthImageViewerPage *self,
 		return;
 
 	if (add_to_history)
-		gth_image_history_add_image (self->priv->history, image, -1, TRUE);
+		gth_image_history_add_surface (self->priv->history, image, -1, TRUE);
 
-	_gth_image_viewer_page_set_image (self, image, -1, TRUE);
+	_gth_image_viewer_page_set_surface (self, image, -1, TRUE);
 
 	if (add_to_history)
 		gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
@@ -1940,6 +1969,7 @@ gth_image_viewer_page_get_original (GthImageViewerPage	 *self,
 						  gth_image_viewer_page_get_original);
 	data->cancellable = (cancellable != NULL) ? g_object_ref (cancellable) : g_cancellable_new ();
 
+	_gth_image_preloader_init_preloader (self);
 	gth_image_preloader_load (self->priv->preloader,
 				  self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
 				  GTH_ORIGINAL_SIZE,
@@ -2068,3 +2098,27 @@ gth_original_image_task_get_image (GthTask *task)
 {
 	return gth_image_task_get_destination_surface (GTH_IMAGE_TASK (task));
 }
+
+
+void
+gth_image_viewer_page_apply_icc_profile	(GthImageViewerPage *self,
+					 gboolean            apply)
+{
+	GthFileData *file_data;
+
+	self->priv->apply_icc_profile = apply;
+	gth_image_preloader_clear_cache (self->priv->preloader);
+
+	file_data = gth_browser_get_current_file (self->priv->browser);
+	if (file_data == NULL)
+		return;
+
+	/* force a complete reload */
+	_g_object_unref (self->priv->last_loaded);
+	self->priv->last_loaded = NULL;
+
+	g_object_ref (file_data);
+	_gth_image_viewer_page_load (self, file_data);
+
+	g_object_unref (file_data);
+}
