@@ -40,13 +40,6 @@
 #define CARDINALITY_FORMAT "(%d)"
 
 
-enum {
-	EXTENSION_DESCRIPTION_COLUMN,
-	EXTENSION_ORIGINAL_STATUS_COLUMN,
-	EXTENSION_COLUMNS
-};
-
-
 enum {
 	CATEGORY_ID_COLUMN,
 	CATEGORY_NAME_COLUMN,
@@ -164,43 +157,6 @@ list_equal (GList *list1,
 }
 
 
-static void
-extension_description_data_func_cb (GtkTreeViewColumn *tree_column,
-				    GtkCellRenderer   *cell,
-				    GtkTreeModel      *tree_model,
-				    GtkTreeIter       *iter,
-				    gpointer           user_data)
-{
-	GthExtensionDescription *description;
-	char                    *text;
-
-	gtk_tree_model_get (tree_model, iter, EXTENSION_DESCRIPTION_COLUMN, &description, -1);
-
-	text = g_markup_printf_escaped ("<b>%s</b>\n<small>%s</small>", description->name, description->description);
-	g_object_set (G_OBJECT (cell), "markup", text, NULL);
-	g_object_set (G_OBJECT (cell), "sensitive", gth_extension_description_is_active (description), NULL);
-
-	g_free (text);
-	g_object_unref (description);
-}
-
-
-static void
-extension_active_data_func_cb (GtkTreeViewColumn *tree_column,
-		 	       GtkCellRenderer   *cell,
-			       GtkTreeModel      *tree_model,
-			       GtkTreeIter       *iter,
-			       gpointer           user_data)
-{
-	GthExtensionDescription *description;
-
-	gtk_tree_model_get (tree_model, iter, EXTENSION_DESCRIPTION_COLUMN, &description, -1);
-	g_object_set (G_OBJECT (cell), "active", gth_extension_description_is_active (description), NULL);
-
-	g_object_unref (description);
-}
-
-
 static void
 add_category_combobox_columns (GtkWidget   *combo_box,
 			       BrowserData *data)
@@ -390,38 +346,6 @@ category_view_separator_func (GtkTreeModel *model,
 }
 
 
-static gboolean
-category_model_visible_func (GtkTreeModel *model,
-			     GtkTreeIter  *iter,
-			     gpointer      user_data)
-{
-	BrowserData             *data = user_data;
-	GthExtensionDescription *description;
-	gboolean                 original_status_is_active;
-	gboolean                 visible;
-
-	if (strcmp (data->current_category, EXTENSION_CATEGORY_ALL) == 0)
-		return TRUE;
-
-	gtk_tree_model_get (model, iter,
-			    EXTENSION_DESCRIPTION_COLUMN, &description,
-			    EXTENSION_ORIGINAL_STATUS_COLUMN, &original_status_is_active,
-			    -1);
-
-	visible = FALSE;
-	if ((strcmp (data->current_category, EXTENSION_CATEGORY_ENABLED) == 0) && original_status_is_active)
-		visible = TRUE;
-	else if ((strcmp (data->current_category, EXTENSION_CATEGORY_DISABLED) == 0) && ! original_status_is_active)
-		visible = TRUE;
-	else
-		visible = g_strcmp0 (description->category, data->current_category) == 0;
-
-	g_object_unref (description);
-
-	return visible;
-}
-
-
 static void
 extension_switch_activated_cb (GObject    *gobject,
 			       GParamSpec *pspec,
@@ -522,6 +446,7 @@ create_extensions_row (GthExtensionDescription	*description,
 	g_object_set_data_full (G_OBJECT (row), "extension-row-data", row_data, (GDestroyNotify) row_data_free);
 
 	row_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (row_box), 10);
 	gtk_container_add (GTK_CONTAINER (row), row_box);
 
 	button = gtk_switch_new ();
@@ -536,13 +461,15 @@ create_extensions_row (GthExtensionDescription	*description,
 	label_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
 	label = gtk_label_new (description->name);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
 	gtk_style_context_add_class (gtk_widget_get_style_context (label), "extension-name");
 	gtk_box_pack_start (GTK_BOX (label_box), label, FALSE, FALSE, 0);
 
 	label = gtk_label_new (description->description);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
 	gtk_style_context_add_class (gtk_widget_get_style_context (label), "extension-description");
 	gtk_box_pack_start (GTK_BOX (label_box), label, FALSE, FALSE, 0);
@@ -551,6 +478,7 @@ create_extensions_row (GthExtensionDescription	*description,
 	extension = gth_extension_description_get_extension (description);
 	if ((extension != NULL) && gth_extension_is_configurable (extension)) {
 		button = gtk_button_new_from_icon_name ("emblem-system-symbolic", GTK_ICON_SIZE_BUTTON);
+		gtk_widget_set_tooltip_text (button, _("Preferences"));
 		gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
 		g_signal_connect (button,
 				  "clicked",
@@ -558,7 +486,7 @@ create_extensions_row (GthExtensionDescription	*description,
 				  row_data);
 		gtk_box_pack_start (GTK_BOX (row_box), button, FALSE, FALSE, 0);
 	}
-	if (extension != NULL) {
+	if ((extension != NULL) && (g_strcmp0 (description->authors[0], _("gthumb development team")) != 0)) {
 		button = gtk_button_new_from_icon_name ("dialog-information-symbolic", GTK_ICON_SIZE_BUTTON);
 		gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
 		g_signal_connect (button,
@@ -618,8 +546,6 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 					  GtkBuilder *dialog_builder)
 {
 	BrowserData          *data;
-	GtkWidget            *notebook;
-	GtkWidget            *page;
 	GthExtensionManager  *manager;
 	GList                *extensions;
 	GList                *descriptions;
@@ -627,6 +553,7 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 	char                **all_active_extensions;
 	int                   i;
 	GtkWidget            *label;
+	GtkWidget            *page;
 
 	data = g_new0 (BrowserData, 1);
 	data->builder = _gtk_builder_new_from_file ("extensions-preferences.ui", NULL);
@@ -652,10 +579,6 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 	data->active_extensions = g_list_reverse (data->active_extensions);
 	g_strfreev (all_active_extensions);
 
-	notebook = _gtk_builder_get_widget (dialog_builder, "notebook");
-	page = _gtk_builder_get_widget (data->builder, "preferences_page");
-	gtk_widget_show (page);
-
 	/* the extensions list */
 
 	data->extensions_list = gtk_list_box_new ();
@@ -681,6 +604,8 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 
 	for (scan = descriptions; scan; scan = scan->next) {
 		GthExtensionDescription *description = scan->data;
+		if (scan != descriptions)
+			_gtk_list_box_add_separator (GTK_LIST_BOX (data->extensions_list));
 		gtk_container_add (GTK_CONTAINER (data->extensions_list), create_extensions_row (description, data));
 	}
 
@@ -739,7 +664,9 @@ extensions__dlg_preferences_construct_cb (GtkWidget  *dialog,
 	label = gtk_label_new (_("Extensions"));
 	gtk_widget_show (label);
 
-	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+	page = _gtk_builder_get_widget (data->builder, "preferences_page");
+	gtk_widget_show (page);
+	gtk_notebook_append_page (GTK_NOTEBOOK (_gtk_builder_get_widget (dialog_builder, "notebook")), page, label);
 
 	g_object_set_data_full (G_OBJECT (dialog), BROWSER_DATA_KEY, data, (GDestroyNotify) browser_data_free);
 }
