@@ -865,6 +865,62 @@ _g_replace_pattern (const char *utf8_text,
 }
 
 
+int
+_g_utf8_first_ascii_space (const char *str)
+{
+	const char *pos;
+
+	pos = str;
+	while (pos != NULL) {
+		gunichar c = g_utf8_get_char (pos);
+		if (c == 0)
+			break;
+		if (g_ascii_isspace (c))
+			return g_utf8_pointer_to_offset (str, pos);
+		pos = g_utf8_next_char (pos);
+	}
+
+	return -1;
+}
+
+
+gboolean
+_g_utf8_has_prefix (const char  *string,
+		    const char  *prefix)
+{
+	char     *substring;
+	gboolean  result;
+
+	if (string == NULL)
+		return FALSE;
+	if (prefix == NULL)
+		return TRUE;
+
+	substring = g_utf8_substring (string, 0, g_utf8_strlen (prefix, -1));
+	if (substring == NULL)
+		return FALSE;
+
+	result = g_utf8_collate (substring, prefix) == 0;
+	g_free (substring);
+
+	return result;
+}
+
+
+char *
+_g_utf8_remove_prefix (const char *string,
+		       int         prefix_length)
+{
+	int str_length;
+
+	str_length = g_utf8_strlen (string, -1);
+	if (str_length <= prefix_length)
+		return NULL;
+
+	return g_utf8_substring (string, prefix_length, str_length);
+}
+
+
 char *
 _g_utf8_replace (const char  *string,
 		 const char  *pattern,
@@ -1029,6 +1085,39 @@ _g_utf8_remove_extension (const char *str)
 }
 
 
+char *
+_g_utf8_try_from_any (const char *str)
+{
+	char *utf8_str;
+
+	if (str == NULL)
+		return NULL;
+
+	if (! g_utf8_validate (str, -1, NULL))
+		utf8_str = g_locale_to_utf8 (str, -1, NULL, NULL, NULL);
+	else
+		utf8_str = g_strdup (str);
+
+	return utf8_str;
+}
+
+
+char *
+_g_utf8_from_any (const char *str)
+{
+	char *utf8_str;
+
+	if (str == NULL)
+		return NULL;
+
+	utf8_str = _g_utf8_try_from_any (str);
+	if (utf8_str == NULL)
+		utf8_str = g_strdup (_("(invalid value)"));
+
+	return utf8_str;
+}
+
+
 static int
 remove_from_file_list_and_get_position (GList **file_list,
 					GFile  *file)
@@ -3008,13 +3097,44 @@ _g_settings_get_uri (GSettings  *settings,
 		return NULL;
 	}
 
-	uri = _g_replace (value, "~", g_get_home_dir ());
+	uri = _g_replace (value, "file://~", get_home_uri ());
+
 	g_free (value);
 
 	return uri;
 }
 
 
+char *
+_g_settings_get_uri_or_special_dir (GSettings      *settings,
+		     	     	    const char     *key,
+		     	     	    GUserDirectory  directory)
+{
+	char *uri;
+
+	uri = g_settings_get_string (settings, key);
+	if ((uri == NULL) || (strcmp (uri, "~") == 0) || (strcmp (uri, "file://~") == 0)) {
+		const char *dir;
+
+		g_free (uri);
+		uri = NULL;
+
+		dir = g_get_user_special_dir (directory);
+		if (dir != NULL)
+			uri = g_filename_to_uri (dir, NULL, NULL);
+		if (uri == NULL)
+			uri = g_strdup (get_home_uri ());
+	}
+	else {
+		char *tmp = uri;
+		uri = _g_replace (tmp, "file://~", get_home_uri ());
+		g_free (tmp);
+	}
+
+	return uri;
+}
+
+
 void
 _g_settings_set_uri (GSettings  *settings,
 		     const char *key,
@@ -3137,3 +3257,17 @@ _g_error_free (GError *error)
 	if (error != NULL)
 		g_error_free (error);
 }
+
+
+void
+toggle_action_activated (GSimpleAction *action,
+			 GVariant      *parameter,
+			 gpointer       data)
+{
+	GVariant *state;
+
+	state = g_action_get_state (G_ACTION (action));
+	g_action_change_state (G_ACTION (action), g_variant_new_boolean (! g_variant_get_boolean (state)));
+
+	g_variant_unref (state);
+}
