@@ -180,6 +180,11 @@ char *          _g_replace                       (const char  *str,
 char *          _g_replace_pattern               (const char  *utf8_text,
 						  gunichar     pattern,
 						  const char  *value);
+int             _g_utf8_first_ascii_space        (const char  *string);
+gboolean        _g_utf8_has_prefix               (const char  *string,
+						  const char  *prefix);
+char *		_g_utf8_remove_prefix		 (const char  *string,
+						  int         prefix_length);
 char *          _g_utf8_replace                  (const char  *string,
 						  const char  *pattern,
 						  const char  *replacement);
@@ -191,6 +196,8 @@ char **         _g_utf8_strsplit                 (const char  *string,
 char *          _g_utf8_strstrip                 (const char  *str);
 gboolean        _g_utf8_all_spaces               (const char  *utf8_string);
 char *          _g_utf8_remove_extension         (const char  *str);
+char *          _g_utf8_try_from_any             (const char  *str);
+char *          _g_utf8_from_any                 (const char  *str);
 GList *         _g_list_insert_list_before       (GList       *list1,
 						  GList       *sibling,
 						  GList       *list2);
@@ -327,6 +334,10 @@ gboolean        _g_mime_type_is_audio            (const char *mime_type);
 
 char *          _g_settings_get_uri              (GSettings  *settings,
 						  const char *key);
+char *          _g_settings_get_uri_or_special_dir
+						 (GSettings      *settings,
+						  const char     *key,
+						  GUserDirectory  directory);
 void            _g_settings_set_uri              (GSettings  *settings,
 						  const char *key,
 						  const char *uri);
@@ -345,6 +356,8 @@ GList *         _g_list_prepend_link             (GList      *list,
 						  GList      *link);
 void            _g_error_free                    (GError     *error);
 
+DEF_ACTION_CALLBACK (toggle_action_activated)
+
 G_END_DECLS
 
 #endif /* _GLIB_UTILS_H */
