@@ -25,6 +25,7 @@
 #include "gth-file-source.h"
 #include "gth-file-store.h"
 #include "gth-icon-cache.h"
+#include "gth-image.h"
 #include "gth-menu-manager.h"
 #include "gth-task.h"
 #include "gth-window.h"
@@ -57,6 +58,8 @@ G_BEGIN_DECLS
 #define GTH_BROWSER_MENU_MANAGER_FOLDER_FOLDER_ACTIONS		"folder.folder-actions"
 #define GTH_BROWSER_MENU_MANAGER_FOLDER_OTHER_ACTIONS		"folder.other-actions"
 
+#define GTH_BROWSER_HEADER_BAR_BIG_MARGIN 12
+
 #define GTH_TYPE_BROWSER              (gth_browser_get_type ())
 #define GTH_BROWSER(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTH_TYPE_BROWSER, GthBrowser))
 #define GTH_BROWSER_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GTH_BROWSER_TYPE, GthBrowserClass))
@@ -88,6 +91,9 @@ typedef enum { /*< skip >*/
 	GTH_BROWSER_HEADER_SECTION_VIEWER_VIEW,
 	GTH_BROWSER_HEADER_SECTION_VIEWER_EDIT_SIDEBAR,
 	GTH_BROWSER_HEADER_SECTION_VIEWER_EDIT,
+	GTH_BROWSER_HEADER_SECTION_EDITOR_NAVIGATION,
+	GTH_BROWSER_HEADER_SECTION_EDITOR_COMMANDS,
+	GTH_BROWSER_HEADER_SECTION_EDITOR_APPLY,
 	GTH_BROWSER_N_HEADER_SECTIONS
 } GthBrowserHeaderSection;
 
@@ -159,6 +165,13 @@ GtkWidget *	 gth_browser_add_header_bar_toggle_button
 						     const char			*tooltip,
 						     const char 		*action_name,
 						     const char			*accelerator);
+GtkWidget *	 gth_browser_add_header_bar_label_button
+						    (GthBrowser			*browser,
+						     GthBrowserHeaderSection	 section,
+						     const char			*label,
+						     const char			*tooltip,
+						     const char			*action_name,
+						     const char			*accelerator);
 void		 gth_browser_add_menu_manager_for_menu
 						    (GthBrowser			*browser,
 						     const char			*menu_id,
@@ -248,6 +261,8 @@ void             gth_browser_ask_whether_to_save    (GthBrowser       *browser,
 				 	 	     gpointer          user_data);
 void             gth_browser_save_state             (GthBrowser       *browser);
 gboolean         gth_browser_restore_state          (GthBrowser       *browser);
+void		 gth_browser_apply_editor_changes   (GthBrowser	      *browser);
+GthICCProfile    gth_browser_get_screen_profile     (GthBrowser	      *browser);
 
 /* utilities */
 
