@@ -914,7 +914,6 @@ scroll_to (GthImageViewer *self,
 
 	window = gtk_widget_get_window (GTK_WIDGET (self));
 	gdk_window_scroll (window, -delta_x, -delta_y);
-	gdk_window_process_updates (window, TRUE);
 }
 
 
@@ -1780,6 +1779,8 @@ gth_image_viewer_set_image (GthImageViewer *self,
 	if (self->priv->image != image)
 		_g_clear_object (&self->priv->image);
 
+	self->priv->image = g_object_ref (image);
+
 	if (gth_image_get_is_animation (image)) {
 		GdkPixbufAnimation *animation;
 
@@ -1792,7 +1793,6 @@ gth_image_viewer_set_image (GthImageViewer *self,
 		cairo_surface_t *surface;
 
 		if (gth_image_get_is_zoomable (image)) {
-			self->priv->image = g_object_ref (image);
 			gth_image_set_zoom (self->priv->image,
 					    self->priv->zoom_level,
 					    &original_width,
@@ -2095,7 +2095,7 @@ gth_image_viewer_get_zoom_quality_filter (GthImageViewer *viewer)
 	else
 		filter = CAIRO_FILTER_BEST;
 
-	if (gth_image_viewer_get_zoom (viewer) == 1.0)
+	if (gth_image_viewer_get_zoom (viewer) >= 1.0)
 		filter = CAIRO_FILTER_FAST;
 
 	return filter;
