@@ -26,7 +26,8 @@
 
 #define DIALOG_WIDTH 450
 #define SHOW_DELAY 500
-#define PULSE_INTERVAL 500
+#define PULSE_INTERVAL (1000 / 12)
+#define PULSE_STEP (1.0 / 20.0)
 
 
 /* -- gth_task_progress -- */
@@ -43,7 +44,7 @@ typedef struct _GthTaskProgress GthTaskProgress;
 typedef struct _GthTaskProgressClass GthTaskProgressClass;
 
 struct _GthTaskProgress {
-	GtkBox    parent_instance;
+	GtkBox     parent_instance;
 	GthTask   *task;
 	GtkWidget *description_label;
 	GtkWidget *details_label;
@@ -59,6 +60,9 @@ struct _GthTaskProgressClass {
 };
 
 
+GType gth_task_progress_get_type (void);
+
+
 G_DEFINE_TYPE (GthTaskProgress, gth_task_progress, GTK_TYPE_BOX)
 
 
@@ -121,6 +125,7 @@ gth_task_progress_init (GthTaskProgress *self)
 	gtk_box_pack_start (GTK_BOX (vbox), self->description_label, FALSE, FALSE, 0);
 
 	self->fraction_progressbar = gtk_progress_bar_new ();
+	gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (self->fraction_progressbar), PULSE_STEP);
 	gtk_widget_set_size_request (self->fraction_progressbar, -1, 15);
 	gtk_widget_show (self->fraction_progressbar);
 	gtk_box_pack_start (GTK_BOX (vbox), self->fraction_progressbar, FALSE, FALSE, 0);
@@ -144,7 +149,7 @@ gth_task_progress_init (GthTaskProgress *self)
 	gtk_widget_set_tooltip_text (self->cancel_button, _("Cancel operation"));
 	gtk_box_pack_start (GTK_BOX (vbox), self->cancel_button, TRUE, FALSE, 0);
 
-	image = gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name ("edit-delete-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show (image);
 	gtk_container_add (GTK_CONTAINER (self->cancel_button), image);
 
@@ -296,7 +301,7 @@ gth_progress_dialog_init (GthProgressDialog *self)
 	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), 5);
 	gtk_container_set_border_width (GTK_CONTAINER (self), 5);
 
-	gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+	gtk_dialog_add_button (GTK_DIALOG (self), _GTK_LABEL_CLOSE, GTK_RESPONSE_CLOSE);
 
 	self->priv->task_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show (self->priv->task_box);
@@ -340,10 +345,10 @@ _show_dialog_cb (gpointer data)
 
 
 static void
-task_dialog_cb (GthTask   *task,
-		gboolean   opened,
-		GtkWidget *dialog,
-		gpointer   user_data)
+progress_dialog_task_dialog_cb (GthTask   *task,
+				gboolean   opened,
+				GtkWidget *dialog,
+				gpointer   user_data)
 {
 	GthProgressDialog *self = user_data;
 
@@ -364,12 +369,12 @@ task_dialog_cb (GthTask   *task,
 
 
 static void
-main_dialog_task_progress_cb (GthTask    *task,
-			      const char *description,
-			      const char *details,
-			      gboolean    pulse,
-			      double      fraction,
-			      gpointer    user_data)
+progress_dialog_task_progress_cb (GthTask    *task,
+				  const char *description,
+				  const char *details,
+				  gboolean    pulse,
+				  double      fraction,
+				  gpointer    user_data)
 {
 	GthProgressDialog *self = user_data;
 	GString           *title;
@@ -392,6 +397,20 @@ main_dialog_task_progress_cb (GthTask    *task,
 }
 
 
+static void
+progress_dialog_task_completed_cb (GthTask  *task,
+		   	   	   GError   *error,
+		   	   	   gpointer  user_data)
+{
+	GthProgressDialog *self = user_data;
+
+	if ((error != NULL) && ! g_error_matches (error, GTH_TASK_ERROR, GTH_TASK_ERROR_CANCELLED))
+		_gtk_error_dialog_from_gerror_show (self->priv->parent,
+						    _("Could not perform the operation"),
+						    error);
+}
+
+
 void
 gth_progress_dialog_add_task (GthProgressDialog *self,
 			      GthTask           *task)
@@ -400,11 +419,15 @@ gth_progress_dialog_add_task (GthProgressDialog *self,
 
 	g_signal_connect (task,
 			  "dialog",
-			  G_CALLBACK (task_dialog_cb),
+			  G_CALLBACK (progress_dialog_task_dialog_cb),
 			  self);
 	g_signal_connect (task,
 			  "progress",
-			  G_CALLBACK (main_dialog_task_progress_cb),
+			  G_CALLBACK (progress_dialog_task_progress_cb),
+			  self);
+	g_signal_connect (task,
+			  "completed",
+			  G_CALLBACK (progress_dialog_task_completed_cb),
 			  self);
 
 	gtk_window_set_title (GTK_WINDOW (self), "");
