@@ -61,6 +61,14 @@ GtkWidget *     _gtk_yesno_dialog_new                      (GtkWindow        *pa
 							    const char       *message,
 							    const char       *no_button_text,
 							    const char       *yes_button_text);
+gchar *		_gtk_request_dialog_run			   (GtkWindow        *parent,
+							    GtkDialogFlags    flags,
+							    const char       *title,
+							    const char       *message,
+							    const char       *default_value,
+							    int               max_length,
+							    const char       *no_button_text,
+							    const char       *yes_button_text);
 void            _gtk_error_dialog_from_gerror_run          (GtkWindow        *parent,
 					    	    	    const char       *title,
 					    	    	    GError           *gerror);
@@ -146,6 +154,7 @@ void		_gtk_window_add_accelerators_from_menu	   (GtkWindow		*window,
 							    GMenuModel		*menu);
 gboolean	_gtk_window_get_is_maximized 		   (GtkWindow		*window);
 GdkDevice *     _gtk_widget_get_client_pointer		   (GtkWidget		*widget);
+void            _gtk_list_box_add_separator		   (GtkListBox		*list_box);
 
 G_END_DECLS
 
